import csv
import json
from slugify import UniqueSlugify
from .adapter import Adapter as BaseAdapter
from ..constants import (ALLOWED_HTML_TAGS, HTML_FIELDS,
                         WOO_IMP_EXP_PRODUCT_FIELDS)


CATEGORY_DELIMITER = ' / '
CSV_DELIMITER = ';'


class EcwidAdapter(BaseAdapter):
    """Input handler for product data exported from Ecwid.
    See https://www.ecwid.com/
    """

    categories = {}

    def parse(self, filename):
        """Parse an ecwid CSV file."""
        def parse_row(r):
            """Parse a row of the Ecwid CSV file and rearrange the fields for
            WooCommerce."""
            raw_category_names = list(filter(None, [
                r["category{0}".format(c)] if r["category{0}".format(c)]
                else None
                for c in range(1, 3)]))
            product = dict(WOO_IMP_EXP_PRODUCT_FIELDS)
            product.update({
                "Id": r["product_id"],
                "Product Name": r["name"],
                "Categories": raw_category_names,
                "SKU": r["sku"],
                "Weight": r["weight"] if float(r["weight"]) > 0 else '',
                "Product Images": r["image"],
                "Product Image Set": "yes" if r["image"] else "no",
                "Short Description": r["seo_description"],
                "Description": r["description"],
                "Price": r["price"],
                "Regular Price": r["recommended_price"] or r["price"],
                "Visibility": ("visible" if r["enabled"] == "yes"
                               else "invisible"),
                "Purchaseable": r["enabled"] or "no",
            })
            return product

        with open(filename) as csv_in:
            csv_reader = csv.DictReader(csv_in, delimiter=CSV_DELIMITER)
            raw_category_names = []
            for row in csv_reader:
                product = parse_row(row)
                self.products.append(product)
                raw_category_names += product['Categories']

            raw_category_names = list(set(raw_category_names))  # Uniquify
            categories = self._expand_categories(raw_category_names)
            for product in self.products:
                if 'Categories' in product and product['Categories']:
                    product['Categories'] = json.dumps([{
                        'name': categories[c]['Name'],
                        'slug': categories[c]['Slug']}
                        for c in product['Categories']])
            self.categories = categories.values()

    def _expand_categories(self, raw_category_names):
        """Convert a list of category name strings into a list of WooCommerce-ready
        category dicts. Adds parent categories where missing."""
        def expand_category(name, cat_id, slugify, products):
            """Build a rich category entry from minimal input."""
            clean_name = (name[name.rfind(CATEGORY_DELIMITER) +
                               len(CATEGORY_DELIMITER):]
                          if CATEGORY_DELIMITER and CATEGORY_DELIMITER in name
                          else name)
            return {
                'Id': cat_id,
                'Term Taxonomy Id': cat_id,
                'Name': clean_name,
                'Slug': slugify(clean_name),
                'Parent Name': (
                    name[:name.rfind(CATEGORY_DELIMITER)]
                    if CATEGORY_DELIMITER and CATEGORY_DELIMITER in name
                    else ''),
                'Count': sum([1 for p in products if name in p['Categories']]),
            }

        def add_missing_parents(categories, products, slugify):
            """Find categories whose parents do not exist, and recursively add
               those parents to the list."""

            def split_category_name(name):
                """Get the names of this categories ancestors, and itself."""
                return [(name[:name.rfind(CATEGORY_DELIMITER)]
                         if CATEGORY_DELIMITER and CATEGORY_DELIMITER in name
                         else '') for i in range(
                             name.count(CATEGORY_DELIMITER))] + [name]

            cats_with_parents = categories.copy()
            for i, c in categories.items():
                try:
                    for ancestor in split_category_name(c['Parent Name']):
                        if ancestor not in cats_with_parents:
                            cat_id = len(cats_with_parents) + 1
                            cats_with_parents[ancestor] = expand_category(
                                ancestor, cat_id, slugify, products)
                except KeyError:
                    continue
            return cats_with_parents
        categories = {}
        slugify = UniqueSlugify(to_lower=True)
        for idx, name in enumerate(raw_category_names):
            categories[name] = expand_category(name, idx, slugify,
                                               self.products)

        categories = add_missing_parents(categories, self.products, slugify)
        for i, c in categories.items():
            if c['Parent Name']:
                parent = categories[c['Parent Name']]
                c['Parent Id'] = parent['Id']
                c['Parent Slug'] = parent['Slug']
            else:
                c['Parent Id'] = ''
                c['Parent Slug'] = ''
            del c['Parent Name']
        return categories

adapter_cls = EcwidAdapter
